import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Optional, Inject, PLATFORM_ID, Component, ChangeDetectionStrategy, Input, NgModule } from '@angular/core';
import { from, map, BehaviorSubject, Subscription } from 'rxjs';
import { DOCUMENT, isPlatformServer, CommonModule } from '@angular/common';

const SvgOptionsToken = new InjectionToken('__SvgOptionsToken__');

class SvgLoadStrategy {
}
SvgLoadStrategy.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgLoadStrategy, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
SvgLoadStrategy.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgLoadStrategy });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgLoadStrategy, decorators: [{
            type: Injectable
        }] });

// eslint-disable-next-line @typescript-eslint/ban-types
function getZoneUnPatchedApi(name, elem) {
    return elem['__zone_symbol__' + name] !== undefined
        ? elem['__zone_symbol__' + name]
        : elem[name];
}

class SvgLoadStrategyImpl {
    constructor() {
        this.fetch = getZoneUnPatchedApi('fetch', window);
    }
    load(url) {
        return from(fetch(url).then((res) => (!res.ok ? '' : res.text())));
    }
}
SvgLoadStrategyImpl.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgLoadStrategyImpl, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
SvgLoadStrategyImpl.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgLoadStrategyImpl });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgLoadStrategyImpl, decorators: [{
            type: Injectable
        }] });

const suspenseSvg = `<svg
      viewBox="0 0 24 24"
      fill="currentColor"
      xmlns="http://www.w3.org/2000/svg"
    >
      <path d="M0 0h24v24H0V0z" fill="none" />
      <circle cx="15.5" cy="9.5" r="1.5" />
      <circle cx="8.5" cy="9.5" r="1.5" />
      <path
        d="M11.99 2C6.47 2 2 6.48 2 12s4.47 10 9.99 10C17.52 22 22 17.52 22 12S17.52 2 11.99 2zM12 20c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm0-6c-2.33 0-4.32 1.45-5.12 3.5h1.67c.69-1.19 1.97-2 3.45-2s2.75.81 3.45 2h1.67c-.8-2.05-2.79-3.5-5.12-3.5z"
      />
    </svg>`;

// @TODO compose svgs in 1 sprite and fetch by id as before
const element$1 = undefined;
function createDomParser(document) {
    const e = element$1 || document.createElement('DIV');
    return (s) => {
        e && (e.innerHTML = s);
        return e.firstChild;
    };
}
function styleDomCacheForPerformance(el) {
    /**
     * reduce paint area with with/height 0 and overflow hidden
     * fixed position of -2000px to always have it offscreen and outside of any native trigger (viewport observer in content visibilities)
     * contain:content to leverage css perf features for older browsers not supporting content-visibility
     * content-visibility: auto to exclude it completely from styles recalculation
     */
    el.setAttribute('style', `
    overflow: hidden;
    width: 0px;
    height: 0px;
    position: fixed;
    bottom: -2000px;
    contain: content;
    content-visibility: auto;
  `);
    return el;
}
class SvgRegistry {
    constructor(document, svgLoadStrategy, svgOptions) {
        this.document = document;
        this.svgLoadStrategy = svgLoadStrategy;
        this.svgOptions = svgOptions;
        this.domParser = createDomParser(this.document);
        this.svgDomCache = (() => {
            // The DOM cache could be already created on SSR or due to multiple instances of the registry
            const domCache = this.document.getElementById('svg-cache') ||
                this.domParser('<div id="svg-cache"></div>');
            styleDomCacheForPerformance(domCache);
            this.document.body.appendChild(domCache);
            return domCache;
        })();
        this._svgHrefCache = new Map();
        this._cachedSvgs = new Set();
        this.defaultSize = this.svgOptions?.defaultSize || '24';
        this._defaultViewBox = `0 0 ${this.defaultSize} ${this.defaultSize}`;
        this.url = this.svgOptions.url;
        this.fetchSvg = (svgName) => {
            const svgId = this.svgId(svgName);
            // if the svg is already fetched we return early
            if (this._cachedSvgs.has(svgId)) {
                return;
            }
            this._cachedSvgs.add(svgId);
            // trigger fetch
            this.svgLoadStrategy
                .load(this.svgOptions.url(svgName))
                .subscribe((body) => this.cacheSvgInDOM(svgId, body), console.error);
        };
        // configure suspense svg
        const suspenseSvgId = this.svgId('suspense');
        !this._cachedSvgs.has(suspenseSvgId) &&
            this.cacheSvgInDOM(suspenseSvgId, this.svgOptions.suspenseSvgString || suspenseSvg);
        this.hydrateFromDom();
    }
    hydrateFromDom() {
        // hydrate DOM cache
        Array.from(this.svgDomCache.children).forEach((i) => {
            // add to fetchedSvgs
            this._cachedSvgs.add(i.id);
            // i.
            // publish to components and render it
            this.getSvgSubject(i.id).next({
                name: i.id,
                viewBox: i.getAttribute('viewBox') || this._defaultViewBox,
            });
        });
    }
    isSvgCached(name) {
        return this._cachedSvgs.has(this.svgId(name));
    }
    svgCache$(name) {
        // start by displaying the suspense svg immediately
        return this.getSvgSubject(this.svgId(name)).pipe(map((svg) => ({ name: `#${svg.name}`, viewBox: svg.viewBox })));
    }
    cacheSvgInDOM(svgId, svgString) {
        // create HTML
        const svgElem = this.domParser(svgString);
        // parse the svg string and get a viewBox
        const viewBox = svgString.match(/viewBox="([^"]*)"/);
        // the SVG element needs to be accessible over a href and end with a specific anchor to select the element by id
        svgElem?.setAttribute && svgElem.setAttribute('id', svgId);
        this.svgDomCache.appendChild(svgElem);
        // notify subscribers about change
        this.getSvgSubject(svgId).next({
            name: svgId,
            viewBox: viewBox?.[1] || this._defaultViewBox,
        });
    }
    /*
    private removeSvgInDOM(svgId: string): void {
      this.svgDomCache.querySelector(`#${svgId}`)?.remove();
    }*/
    getSvgSubject(svgId) {
        if (!this._svgHrefCache.has(svgId)) {
            this._svgHrefCache.set(svgId, new BehaviorSubject({
                name: this.svgId('suspense'),
                viewBox: this._defaultViewBox,
            }));
        }
        return this._svgHrefCache.get(svgId);
    }
    // pattern has to be `<svgName>`
    svgId(name) {
        return `${name}`;
    }
}
SvgRegistry.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgRegistry, deps: [{ token: DOCUMENT, optional: true }, { token: SvgLoadStrategy, optional: true }, { token: SvgOptionsToken, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
SvgRegistry.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgRegistry });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: SvgRegistry, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: SvgLoadStrategy, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [SvgLoadStrategy]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [SvgOptionsToken]
                }] }]; } });

/* eslint-disable @typescript-eslint/ban-types */
/**
 * getZoneUnPatchedApi
 *
 * @description
 *
 * This function returns the zone un-patched API for the a specific Browser API.
 * If no element is passed the window is used instead
 *
 * @param name {string} - The name of the API to check.
 * @param elem {any} - The elem to get un-patched API from.
 * @return {Function} - The zone un-patched API in question.
 *
 */
const addEventListener = (elem, name, listener) => getZoneUnPatchedApi('addEventListener', elem).bind(elem)(name, listener);
let element = undefined;
function createGetImgFn(renderer) {
    if (element === undefined) {
        element = renderer.createElement('img');
        element.setAttribute('style', 'display: none; contain: content; content-visibility: auto;');
        element.setAttribute('loading', 'lazy');
        element.setAttribute('width', '0');
        element.setAttribute('height', '0');
    }
    return (src) => {
        const e = element.cloneNode(true);
        e.setAttribute('src', src);
        return e;
    };
}
/**
 * ngx-fast-svg enables lazy loading features of the browser for SVG.
 * It also manages the caching of SVG's in the DOM for multiple usage and different stylings
 * This component also Supports lazy loading with SSR and http transfer cache
 * */
class FastSvgComponent {
    constructor(platform, renderer, registry, element) {
        this.platform = platform;
        this.renderer = renderer;
        this.registry = registry;
        this.element = element;
        this.sub = new Subscription();
        this.getImg = createGetImgFn(this.renderer);
        this.name = '';
        this.size = this.registry.defaultSize;
        this.width = '';
        this.height = '';
        // When the browser loaded the svg resource we trigger the caching mechanism
        // re-fetch -> cache-hit -> get SVG -> cache in DOM
        this.loadedListener = () => {
            this.registry.fetchSvg(this.name);
        };
    }
    ngAfterViewInit() {
        if (!this.name) {
            throw new Error('svg component needs a name to operate');
        }
        // Setup view refs and init them
        const elem = this.element.nativeElement;
        const svg = elem.querySelector('svg');
        // apply size
        if (this.size && svg) {
            // We apply fixed dimensions
            // Additionally to SEO rules, to avoid any scroll flicker caused by `content-visibility:auto` defined in component styles
            svg.setAttribute('width', this.width || this.size);
            svg.setAttribute('height', this.height || this.width || this.size);
        }
        let img = null;
        // if svg is not in cache we append
        if (!this.registry.isSvgCached(this.name)) {
            /**
             CSR - Browser native lazy loading hack
      
             We use an img element here to leverage the browsers native features:
             - lazy loading (loading="lazy") to only load the svg that are actually visible
             - priority hints to down prioritize the fetch to avoid delaying the LCP
      
             While the SVG is loading we display a fallback SVG.
             After the image is loaded we remove it from the DOM. (IMG load event)
             When the new svg arrives we append it to the template.
      
             Note:
             - the image is styled with display none. this prevents any loading of the resource ever.
             on component bootstrap we decide what we want to do. when we remove display none it performs the browser native behavior
             - the image has 0 height and with and containment as well as contnet-visibility to recuce any performance impact
      
      
             Edge cases:
             - only resources that are not loaded in the current session of the browser will get lazy loaded (same URL to trigger loading is not possible)
             - already loaded resources will get emitted from the cache immediately, even if loading is set to lazy :o
             - the image needs to have display other than none
      
             */
            const i = this.getImg(this.registry.url(this.name));
            this.renderer.appendChild(this.element.nativeElement, i);
            // get img
            img = elem.querySelector('img');
            addEventListener(img, 'load', this.loadedListener);
        }
        // Listen to svg changes
        // This potentially could already receive the svg from the cache and drop the img from the DOM before it gets activated for lazy loading.
        // NOTICE:
        // If the svg is already cached the following code will execute synchronously. This gives us the chance to add
        this.sub = this.registry.svgCache$(this.name).subscribe((cache) => {
            // The first child is the `use` tag. The value of href gets displayed as SVG
            svg.children[0].setAttribute('href', cache.name);
            svg.setAttribute('viewBox', cache.viewBox);
            // early exvit no image
            if (!img)
                return;
            // If the img is present
            // and the name in included in the href (svg is fully loaded, not only the suspense svg)
            // Remove the element from the DOM as it is no longer needed
            if (cache.name.includes(this.name)) {
                img.removeEventListener('load', this.loadedListener);
                // removeEventListener.bind(img, 'load', this.loadedListener);
                img.remove();
            }
        });
        // SSR
        if (isPlatformServer(this.platform)) {
            // if SSR load svgs on server => ends up in DOM cache and ships to the client
            this.registry.fetchSvg(this.name);
        }
        // CSR
        else {
            // Activate the lazy loading hack
            // Loading is triggered in the template over loading="lazy" and onload
            // Than the same image is fetched over fromFetch and rendered as SVG. (This will result in a cache hit for this svg)
            //
            // If the img is present activate it
            img && img.style.setProperty('display', 'block');
        }
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
        this.element.nativeElement
            .querySelector('img')
            ?.removeEventListener('load', this.loadedListener);
    }
}
FastSvgComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: FastSvgComponent, deps: [{ token: PLATFORM_ID }, { token: i0.Renderer2 }, { token: SvgRegistry }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component });
FastSvgComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.4", type: FastSvgComponent, selector: "fast-svg", inputs: { name: "name", size: "size", width: "width", height: "height" }, ngImport: i0, template: `
    <!-- Svg - Displayed lazy or at bootstrap time from cache due to SSR or already loaded resources
    We use a DOM caching mechanism to display svg over the 'use' tag and an href attribute.
    - We display an empty SVG at the beginning. Invisible and without dimensions
    - On View init the size is applied even if no svg is loaded to avoid flickering in dimensions
    - A suspense svg is displayed at the same time to reduce visual flickering
    - when the real svg is loaded it is directly cached in the DOM and displayed over a new href value
     -->
    <svg class="fast-svg">
      <use></use>
    </svg>
  `, isInline: true, styles: [":host{display:contents}.fast-svg{margin:3px;contain:content;content-visibility:auto}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: FastSvgComponent, decorators: [{
            type: Component,
            args: [{
                    // eslint-disable-next-line @angular-eslint/component-selector
                    selector: 'fast-svg',
                    template: `
    <!-- Svg - Displayed lazy or at bootstrap time from cache due to SSR or already loaded resources
    We use a DOM caching mechanism to display svg over the 'use' tag and an href attribute.
    - We display an empty SVG at the beginning. Invisible and without dimensions
    - On View init the size is applied even if no svg is loaded to avoid flickering in dimensions
    - A suspense svg is displayed at the same time to reduce visual flickering
    - when the real svg is loaded it is directly cached in the DOM and displayed over a new href value
     -->
    <svg class="fast-svg">
      <use></use>
    </svg>
  `,
                    styles: [
                        `
      :host {
        display: contents;
      }

      .fast-svg {
        margin: 3px;
        /* leverage css perf features for older browsers not supporting content-visibility */
        contain: content;
        /* leverage contain:content improvements, exclude node completely from recalc styles if offscreen */
        content-visibility: auto;
      }
    `,
                    ],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }, { type: i0.Renderer2 }, { type: SvgRegistry }, { type: i0.ElementRef }]; }, propDecorators: { name: [{
                type: Input
            }], size: [{
                type: Input
            }], width: [{
                type: Input
            }], height: [{
                type: Input
            }] } });

class FastSvgModule {
    static forRoot(providers) {
        const svgOptions = {
            url: providers.url,
        };
        providers?.suspenseSvgString &&
            (svgOptions.suspenseSvgString = providers.suspenseSvgString);
        providers?.defaultSize && (svgOptions.defaultSize = providers.defaultSize);
        return {
            ngModule: FastSvgModule,
            providers: [
                {
                    provide: SvgLoadStrategy,
                    useClass: providers.svgLoadStrategy || SvgLoadStrategyImpl,
                },
                {
                    provide: SvgOptionsToken,
                    useValue: svgOptions,
                },
                SvgRegistry,
            ],
        };
    }
    static forChild() {
        return {
            ngModule: FastSvgModule,
            providers: [],
        };
    }
}
FastSvgModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: FastSvgModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
FastSvgModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: FastSvgModule, declarations: [FastSvgComponent], imports: [CommonModule], exports: [FastSvgComponent] });
FastSvgModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: FastSvgModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.4", ngImport: i0, type: FastSvgModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [FastSvgComponent],
                    exports: [FastSvgComponent],
                }]
        }] });

// tokens

/**
 * Generated bundle index. Do not edit.
 */

export { FastSvgComponent, FastSvgModule, SvgLoadStrategy, SvgLoadStrategyImpl, SvgOptionsToken, SvgRegistry };
//# sourceMappingURL=ngx-fast-svg.mjs.map
